package gov.va.med.mhv.api.common.service.impl;

import gov.va.med.mhv.api.cache.CacheHandler;
import gov.va.med.mhv.api.common.service.SessionAPIService;
import gov.va.med.mhv.api.enumeration.ErrorEnum;
import gov.va.med.mhv.api.transfer.ClientApplication;
import gov.va.med.mhv.api.transfer.Session;
import gov.va.med.mhv.api.util.ClientApplicationHelper;
import gov.va.med.mhv.api.util.MHVApiUtility;
import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.enumeration.PatientCorrelationStatus;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.TermsConditionsVerHistCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.PatientServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.delegate.TermsConditionsVerHistServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.InPersonAuthentication;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.TermsConditionsVerHist;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;

import java.sql.Timestamp;
import java.util.Collection;
import java.util.Iterator;

import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.apache.axis.utils.StringUtils;
import org.apache.commons.lang.NotImplementedException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.tigris.atlas.service.BooleanServiceResponse;

public class SessionAPIServiceImpl implements SessionAPIService {

	private static final Log LOG = LogFactory.getLog(SessionAPIServiceImpl.class);
	private static final String PATIENT_PATH = "/patient/";
	private static final String EXPIRES_HEADER_KEY = "Expires";
	private static final String TOKEN_HEADER_KEY = "Token";
	private static final String MHV_CORRELATION_ID_HEADER_KEY = "mhvCorrelationId";
	
	
	private HttpHeaders httpHeaders;
	private UriInfo uriInfo;

	/**
	 * Error codes: 102,108,109,110,111,901,129
	 */
	@Override
	public Session checkSession(MessageContext mc) {
		httpHeaders = mc.getHttpHeaders();
		uriInfo = mc.getUriInfo();
		Session session = checkSession();
		
		//THIS IS SO WE CAN CAPTURE THE USER INFORMATION FOR LOGGING
		CacheHandler.getInstance().setSession(session);
		
		return session;
	}
	
	private Session checkSession() {
		Session session = null;

		String token = MHVApiUtility.getValueOfHeaderElement(httpHeaders, TOKEN_HEADER_KEY);
		if (null != token) {
			try {
				session = MHVApiUtility.decrypt(token);
			} catch (Exception e) {
				MHVApiUtility.throwException(ErrorEnum.INVALID_SESSION_TOKEN_109);
			}
			
			//Validate that the session has data before proceeding
			if( session.getExpirationSeconds() < 0 || session.getTimestamp() == null ) {
				MHVApiUtility.throwException(ErrorEnum.INVALID_SESSION_TOKEN_109);
			}
			
			//Check Application Authentication First
			if( !checkClientSourcePermissions(session.getClientApplication()) ) {
				MHVApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
			}
			//Check Application Authorization First
			if( !checkClientAuthorizationPermissions(session.getClientApplication()) ) {
				MHVApiUtility.throwException(ErrorEnum.APPLICATION_AUTHORIZATION_FAILED_102);
			}

			
			// TODO: REDO 
//			if( !checkClientUserPermissions(session.getUserTypeEnum())){
//				SMApiUtility.throwException(ErrorEnum.INVALID_USER_PERMISSIONS_111);
//			}
			
			if (session.isExpired()) {
				MHVApiUtility.throwException(ErrorEnum.EXPIRED_SESSION_TOKEN_110);
			}

			//Load User object ...
			UserProfileServiceDelegate profDelegate = ServiceDelegateFactory.createUserProfileServiceDelegate();
			PatientServiceDelegate patDelegate = ServiceDelegateFactory.createPatientServiceDelegate();
			UserProfileServiceResponse userProfRes = profDelegate.getUserProfileById(session.getUserId());
			MHVApiUtility.throwExceptionOnErrors(ErrorEnum.AUTHENTICATION_SERVICE_ERROR_901, userProfRes);
			UserProfile userProfile = userProfRes.getUserProfile();
			if (userProfile.getLockDate() != null) {
				MHVApiUtility.throwException(ErrorEnum.INELIGIBLE_USER_REASON_BLOCKED_106);
			}
			if(userProfile.getDeactivationReason() != null) {
				MHVApiUtility.throwException(ErrorEnum.USER_ACCOUNT_DEACTIVE_113);
			}
			PatientServiceResponse  patServResponse = patDelegate.getPatientForUser(userProfile);
			Patient patient = patServResponse.getPatient();
			session.setPatient(patient);
			
			
			//Make session available for any caller on this thread
			CacheHandler.getInstance().setSession(session);
			
			
		} else {
			MHVApiUtility.throwException(ErrorEnum.MISSING_SESSION_TOKEN_108);
		}

		return session;
	}

	/**
	 * Session Error Codes Patient:  101,102,103,104,105,106,107,111,132
	 */
	@Override
	public Response createSession( HttpHeaders headers, UriInfo ui) {
		this.setHeaders(headers);
		this.setUi(ui);
		
		boolean isPatient = ui.getRequestUri().getPath().contains(PATIENT_PATH);
				
		if( isPatient ) {
			return checkPatient();
		} else  {
			throw new NotImplementedException();
		}
		
		
	}

	private boolean checkClientSourcePermissions( ClientApplication client) {
		String path = uriInfo.getRequestUri().getPath();
		if (client == null)
			return false;
		for( String permission: client.getPermissions()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}

	private boolean checkClientAuthorizationPermissions( ClientApplication client) {
		String path = uriInfo.getRequestUri().getPath();
		for( String permission: client.getAuthorizations()) {
			if(path.contains(permission)) {
				return true;
			}
		}
		return false;
	}
	
	private boolean checkClientUserPermissions( UserProfile userProfile ) {
		// TODO:
	return true;	
	}
	
	private ClientApplication getclientApplicationUsingHeaderValue() {
		String appToken = MHVApiUtility.getValueOfHeaderElement(httpHeaders, "appToken");
		ClientApplication source = null;
		
		if (StringUtils.isEmpty(appToken)) {
			MHVApiUtility.throwException(ErrorEnum.MISSING_APPLICATION_TOKEN_132);
		} else {
			try {
				source = ClientApplicationHelper.findClientApplication(appToken);
			} catch (Exception e) {
				MHVApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
			}
		}
		
		return source;
	}

	/**
	 * Checks the application token, path authorized for application.
	 * 
	 * Makes sure user is matching requested resource. (patient)
	 * 
	 * @return
	 */
	private Response checkPatient() {
		Session session = null;

		//Check Application Authentication First
		ClientApplication clientApplication = getclientApplicationUsingHeaderValue();
		
		//Check Application Authorization Second
		if( !checkClientSourcePermissions( clientApplication ) ) {
			MHVApiUtility.throwException(ErrorEnum.APPLICATION_AUTHENTICATION_FAILED_101);
		}
		
		// Get Header Attributes and find user checking the active, opted-in, etc.
		String mhvCorrelationId = MHVApiUtility.getValueOfHeaderElement(httpHeaders, MHV_CORRELATION_ID_HEADER_KEY);

		if (StringUtils.isEmpty(mhvCorrelationId)) {
			MHVApiUtility.throwException(ErrorEnum.MISSING_USER_CREDENTIALS_104);
		} else {
			Long cId = null;
			try {
				cId = Long.valueOf(mhvCorrelationId);
			} catch(NumberFormatException e) {
				MHVApiUtility.throwException(ErrorEnum.INVALID_USER_CREDENTIALS_103);
			}
			if (MHVApiUtility.isMhvAuthenticationEnabled() ) {
				authenticateMHVUser();
				
			}
			
			UserProfileServiceDelegate profDelegate = ServiceDelegateFactory.createUserProfileServiceDelegate();
			PatientServiceDelegate patDelegate = ServiceDelegateFactory.createPatientServiceDelegate();
			UserProfileServiceResponse userProfRes = profDelegate.getUserProfileById(cId);
			MHVApiUtility.throwExceptionOnErrors(ErrorEnum.AUTHENTICATION_SERVICE_ERROR_901, userProfRes);
			UserProfile userProfile = userProfRes.getUserProfile();
			PatientServiceResponse  patServResponse = patDelegate.getPatientForUser(userProfile);
			Patient patient = patServResponse.getPatient();
			if(patient == null)
				MHVApiUtility.throwException(ErrorEnum.USER_NOT_FOUND_105);
			
			if (userProfile.getDeactivationReason() != null ) {
				MHVApiUtility.throwException(ErrorEnum.USER_ACCOUNT_DEACTIVE_113);
			}
			
			if (userProfile.getLockDate() != null) {
				MHVApiUtility.throwException(ErrorEnum.INELIGIBLE_USER_REASON_BLOCKED_106);
			}
			if (!hasTermsBeenAccepted(userProfile)) {
				MHVApiUtility.throwException(ErrorEnum.USER_MHV_TC_STALE__114);
			}
			if (userProfile.getDeactivationReason() != null ) {
				MHVApiUtility.throwException(ErrorEnum.USER_ACCOUNT_DEACTIVE_113);
			}


			
			if( !patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED) ){
				MHVApiUtility.throwException(ErrorEnum.INVALID_USER_PERMISSIONS_111);
			}
			
			//USER IS GOOD TO GO!
			session = new Session(userProfile.getId(), clientApplication);
			session.setPatient(patient);
			
			//Making a Facility Update for user.
			InPersonAuthenticationServiceResponse ipaResponse = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate().getIPAPatientById(patient.getId());
			InPersonAuthentication ipa = ipaResponse.getInPersonAuthentication();
    		if(ipa != null){
				if((ipa.getStatus().equals(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED))) &&
						ipa.getPatient().getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED) ||
						((ipa.getStatus().equals(AuthenticationStatus.getEnum(AuthenticationStatus.UNAUTHENTICATED))) &&
						ipa.getPatient().getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED)) ||
						(patient.getMatchedDateTime() == null && (ipa.getStatus().equals(AuthenticationStatus.getEnum(AuthenticationStatus.PENDINGDATAMISMATCH))) &&
		    					ipa.getPatient().getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED)) ||
						((patient.getMatchedDateTime() != null) &&
						ipa.getPatient().getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED)) ){
					PatientServiceResponse ptResponse = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().updateTreatmentFacilitiesForExistingPatients(patient);
					if (LOG.isDebugEnabled()) {
						LOG.debug("Facility Update from aPI for patient Id: "	+ patient.getId());
					}
					MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, ptResponse);
				}
    		}
			
		}
		
		//Make session available for any caller on this thread
		CacheHandler.getInstance().setSession(session);

		String token = null;
		try {
			token = MHVApiUtility.encrypt(session);
		} catch (Exception e) {
			MHVApiUtility.throwException(ErrorEnum.UNABLE_TO_CREATE_TOKEN_107);
		}

		return Response.ok().header(TOKEN_HEADER_KEY, token).header(EXPIRES_HEADER_KEY,session.getFormattedTimestamp()).build();
	}
	public void authenticateMHVUser() {
		String userName = MHVApiUtility.getValueOfHeaderElement(httpHeaders, "userName");
		String password = MHVApiUtility.getValueOfHeaderElement(httpHeaders, "password");
	    BooleanServiceResponse response = ServiceDelegateFactory.createUserServiceDelegate().authenticate(userName, password);
	    MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, response);

	}
	public boolean hasTermsBeenAccepted (UserProfile user) {
		
		TermsConditionsVerHistServiceDelegate delegate = ServiceDelegateFactory.createTermsConditionsVerHistServiceDelegate();
		TermsConditionsVerHistCollectionServiceResponse termsConVerHistResponse = delegate.getAllTermsConditionsVerHist();
		Collection listTCVerHist = termsConVerHistResponse.getTermsConditionsVerHists();
		Iterator it = listTCVerHist.iterator();
		Timestamp latestContent = null;
		while (it.hasNext()) {
			TermsConditionsVerHist verHist = (TermsConditionsVerHist) it.next();
			if (latestContent == null) {
				latestContent = verHist.getUpdatedTime();
				System.out.println("LATEST CONTENT INIT: "+latestContent);
			}	
			if (verHist.getUpdatedTime().after(latestContent))
				latestContent = verHist.getUpdatedTime();
			System.out.println("LATEST CONTENT RE-INIT: "+latestContent);
		}
		boolean accepted = false;
		if ( latestContent != null ) {
			BooleanServiceResponse response = ServiceDelegateFactory.createUserProfileServiceDelegate().
													verifyLatestTermsAccepted(user, latestContent);
			accepted = response.getBoolean();
		}
		if ( latestContent == null ) {
			return true;
		}
		return accepted;
	}
	

	public HttpHeaders getHeaders() {
		return httpHeaders;
	}
	public void setHeaders(HttpHeaders headers) {
		this.httpHeaders = headers;
	}
	public UriInfo getUi() {
		return uriInfo;
	}
	public void setUi(UriInfo ui) {
		this.uriInfo = ui;
	}
	
}
