/**
 *
 */
package gov.va.med.mhv.rxrefill.util;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.config.ConfigurationManager;

import gov.va.med.mhv.core.util.PropertiesUtil;


/**
 * @author Rob Proper
 *
 */
public class RxRefillProperties {

	private static final Log LOG = LogFactory.getLog(RxRefillProperties.class);

	private static final String RESOURCE_NAME = "/rxrefill.properties";

	private static final String MAX_CONSECUTIVE_REQUESTS_PROPERTY =
		"rxrefill.maxConsecutiveRequests";

	private static final String MAX_CONSECUTIVE_FAILURES_PROPERTY =
		"rxrefill.maxConsecutiveFailures";

	private static final String NO_OF_DAYS_TO_SHOW_TRACKING_DETAILS =
		"rxrefill.noOfDaysToShowTrackingDetails";

	private static RxRefillProperties instance = new RxRefillProperties();

	/**
	 * Returns a RxRefillProperties instance.
	 * @return a RxRefillProperties instance.
	 */
	public static final RxRefillProperties getInstance() {
		return instance;
	}

	private final int maxConsecutiveRequests;
	private final int maxConsecutiveFailures;
	private final int noOfDaysToShowTrackingDetails;

	/**
	 * Hide constructor for singleton pattern
	 */
	private RxRefillProperties() {
		final int defaultMaxRequests = 6;
		final int defaultMaxFailures = 3;
		final int defaultNoOfDays = 51;
		int maxRequests = defaultMaxRequests;
		int maxFailures = defaultMaxFailures;
		int noOfDays = defaultNoOfDays;
		try {
			Properties properties = ConfigurationManager.getConfiguration(
				RESOURCE_NAME);
			maxRequests = PropertiesUtil.getIntValue(RESOURCE_NAME, properties,
				MAX_CONSECUTIVE_REQUESTS_PROPERTY, maxRequests);
			maxFailures = PropertiesUtil.getIntValue(RESOURCE_NAME, properties,
				MAX_CONSECUTIVE_FAILURES_PROPERTY, maxFailures);
			noOfDays = PropertiesUtil.getIntValue(RESOURCE_NAME, properties,
				NO_OF_DAYS_TO_SHOW_TRACKING_DETAILS, noOfDays);
		} catch (IllegalStateException e) {
			// ConfigurationManager throws IllegalStateExceptions when
			// something goes wrong reading or finding the properties file
			LOG.warn("Failed to obtain properties from '" + RESOURCE_NAME
				+ "'. Using default values.");
		}
		this.maxConsecutiveRequests = (maxRequests > 0 ) ? maxRequests
			: defaultMaxRequests;
		this.maxConsecutiveFailures = (maxFailures > 0) ? maxFailures
			: defaultMaxFailures;
		this.noOfDaysToShowTrackingDetails = (noOfDays > 0) ? noOfDays
			: defaultNoOfDays;
		if (LOG.isInfoEnabled()) {
			LOG.info("RxRefill properties{"
				+ " MaxRequests:" + this.maxConsecutiveRequests
				+ "; MaxFailures:" + this.maxConsecutiveFailures);
		}
		if (LOG.isInfoEnabled()) {
			LOG.info("RxRefill properties{"
				+ " noOfDaysToShowTrackingDetails:" + this.noOfDaysToShowTrackingDetails);
		}
	}


	/**
	 * Returns the maximum number of consecutive requests that a single patient
	 * can make in a single day.
	 * @return Returns the maximum number of consecutive requests.
	 */
	public int getMaxConsecutiveRequests() {
		return maxConsecutiveRequests;
	}

	/**
	 * Returns the maximum number of consecutive failed requests that a single
	 * patient can have in a single day.
	 * @return Returns the maximum consecutive failed requests.
	 */
	public int getMaxConsecutiveFailures() {
		return maxConsecutiveFailures;
	}

	/**
	 * Returns the maximum number of consecutive requests that a single patient
	 * can make in a single day.
	 * @return Returns the maximum number of consecutive requests.
	 */
	public int getNoOfDaysToShowTrackingDetails() {
		return noOfDaysToShowTrackingDetails;
	}
}
