/**
 * 
 */
package gov.va.med.mhv.rxrefill.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.MessageFactory;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.EntityCollectionServiceResponse;
import org.tigris.atlas.service.ServiceResponse;

import gov.va.med.mhv.rxrefill.enumeration.RequestFunctionEnumeration;
import gov.va.med.mhv.rxrefill.enumeration.RequestStatusEnumeration;
import gov.va.med.mhv.rxrefill.transfer.Request;

/**
 * Provides a work-around the RxRefill service message passing.
 * Message passed back from the RxRefill service are not usuable as
 * message to create notification for the user.  
 * The RxRefill service passed Informational message back to the caller that 
 * contain Request instances. 
 * These request instances then must be used by the caller to determine the 
 * success or failure of request from the different facilities.
 * This class handles the processing of those informational messages and 
 * converting them into proper messages that can be given to the end-user
 * (see {@link #convertMessages(Collection, EntityCollectionServiceResponse)})
 * This class also provides a method to extract and remove these informational
 * message from a ServiceResponse and return them as a list 
 * (see {@link #removeRequestStatusMessages(ServiceResponse)}).
 * @author Rob Proper
 */
public class ServiceResponseUtil {
	
    private static final String KEY_SUCCEEDED = "succeeded";
    private static final String KEY_REJECTED = "rejected";
    private static final String KEY_IN_PROGRESS = "inProgress";
    private static final String KEY_FAILED = "failed";
    private static final String KEY_MAX_EXCEEDED = "maxExceeded";

    private static final String MESSAGE_KEY_REQUEST_STATUS = 
    	"prescriptionservice.request";

	private ServiceResponseUtil() {
		// Hide because utility class. Needs static calls
	}

	/**
	 * Converts the RxRefill messages in the given response to user readable
	 * messages.
	 * @param refillRequests The requests for refills that were made.
	 * @param response The service response instance.
	 * @param messageKeyPrefix Denote the message key prefix for the specific
	 *    application.
	 */
	public static Messages convertMessages(Collection refillRequests,
		EntityCollectionServiceResponse response, String messageKeyPrefix) 
	{
		messageKeyPrefix = (StringUtils.isBlank(messageKeyPrefix)) ? "" 
			: messageKeyPrefix + ((!messageKeyPrefix.endsWith(".")) ? "." : ""); 
		List<Request> requests = mergeRefillWithStatusRequests(
			removeRequestStatusMessages(response), refillRequests);
		Map<String, String> facilityStatuses = createFacilityStatusMap(
			requests);
		Messages messages = createStatusMessages(facilityStatuses, 
			messageKeyPrefix);
	    response.getMessages().addMessages(messages);
	    return messages;
	}
	
	/**
	 * Merges a list of given status and refill requests.
	 * The refill requests take precedence over the status requests. 
	 * @param statusRequests The list of status requests.
	 * @param refillRequests The collection of refill requests.
	 * @return The merged list.
	 */
    @SuppressWarnings("unchecked")
	private static List<Request> mergeRefillWithStatusRequests(
        List<Request> statusRequests, Collection refillRequests) 
    {
    	List<Request> requests = statusRequests;
    	
		if ((refillRequests != null) && (!refillRequests.isEmpty())) {
			requests = new ArrayList<Request>();
		    requests.addAll(refillRequests); // unchecked
		    // Add status requests for those institutions that do not 
		    // have refill requests
		    for (Iterator i = statusRequests.iterator(); i.hasNext();) {
		        Request status = (Request)i.next();
		        if (hasRequestForInstitution(refillRequests, status.
		        	getInstitution()))
		        {
		        	requests.add(status);
		        }
		    }
		}
	    return requests;               
	}
    
    /** 
     * Checks if a request exists for a given institution.
     * @param requests Requests to check
     * @param institution Institution to check against.
     * @return True, if one exists; false otherwise.
     */
    private static boolean hasRequestForInstitution(Collection requests, 
    	gov.va.med.mhv.rxrefill.transfer.Institution institution)
    {
    	boolean found = false;
    	if (institution != null) {
	        for (Iterator j = requests.iterator(); !found && j.hasNext();) {
	        	Request request = (Request) j.next();
	        	found = (request != null) && institution.equals(request.
	        		getInstitution());
	        }
    	}
    	return found;
    }

	private static Map<String, String> createFacilityStatusMap(
		List<Request> requests) 
	{
	    Set<String> succeededFacilities = new HashSet<String>();
	    Set<String> inProgressFacilities = new HashSet<String>();
	    Set<String> rejectedFacilities = new HashSet<String>();
	    Set<String> failedFacilities = new HashSet<String>();
	    Set<String> maxExceededFacilities = new HashSet<String>();
	    
	    for (Iterator i = requests.iterator(); i.hasNext();) {
	        Request request = (Request) i.next();
	        String facility = request.getInstitution().getName();
	        if (hasExceededMaxFailedAttempts(request)) {
	            maxExceededFacilities.add(facility);
	        } else if (isRejected(request)) {
	           rejectedFacilities.add(facility);
	        } else if (hasFailed(request)) {
	            failedFacilities.add(facility);
	        } else if (isInProgress(request)) {
	            inProgressFacilities.add(facility);
	        } else if (isSuccessfull(request)) {
                succeededFacilities.add(facility);
	        }
	    }
	    
	    Map<String, String> facilityStatuses = new HashMap<String, String>();
	    facilityStatuses.put(KEY_SUCCEEDED, toString(succeededFacilities));
	    facilityStatuses.put(KEY_REJECTED, toString(rejectedFacilities));
		facilityStatuses.put(KEY_IN_PROGRESS, toString(inProgressFacilities));
		facilityStatuses.put(KEY_FAILED, toString(failedFacilities));
		facilityStatuses.put(KEY_MAX_EXCEEDED, toString(maxExceededFacilities));
		return facilityStatuses;
	}

	private static boolean hasExceededMaxFailedAttempts(Request request) {
		return (request != null) 
			&& (request.getExceededMaxFailedAttempts() != null)
			&& request.getExceededMaxFailedAttempts().booleanValue();
	}

	private static boolean isRejected(Request request) {
		return (request != null) && RequestStatusEnumeration.REJECTED.equals(
			request.getStatus().getName());
	}

	private static boolean hasFailed(Request request) {
		return (request != null) &&  RequestStatusEnumeration.FAILED.equals(
			request.getStatus().getName());
	}

	private static boolean isInProgress(Request request) {
		return (request != null) && RequestStatusEnumeration.INPROGRESS.equals(
			request.getStatus().getName());
	}

	private static boolean isSuccessfull(Request request) {
		if (request == null) {
			return false;
		}
		if (!RequestStatusEnumeration.SUCCESSFUL.equals(request.getStatus().
			getName()))
		{
			return false;
		}
		if (request.getRequestFunction() == null) { 
			return false;
		}
		return RequestFunctionEnumeration.REFILLPRESCRIPTION.equals(request.
			      getRequestFunction().getName());
		
//		return (request != null) && RequestStatusEnumeration.SUCCESSFUL.equals(
//				  request.getStatus().getName())
//		    && (request.getRequestFunction() != null)
//		    && RequestFunctionEnumeration.REFILLPRESCRIPTION.equals(request.
//			      getRequestFunction().getName());
	}

    private static Messages createStatusMessages(Map<String, String> statuses,
    	String keyPrefix) 
    {
    	Messages messages = MessageFactory.createMessages();
    	if ((statuses == null) || (statuses.isEmpty())) { 
    		return messages;
    	}
        
        for (String key : statuses.keySet()) {
            String facilities = statuses.get(key);
            if (!StringUtils.isBlank(facilities)) {
	            if (KEY_SUCCEEDED.equals(key)) {
	            	addMessage(messages, keyPrefix, MessageKeys.REFILL_SUCCESS, 
	            		facilities);
	            } else if (KEY_IN_PROGRESS.equals(key)) {
	            	addMessage(messages, keyPrefix, MessageKeys.
	            		REFILL_INPROGRESS, facilities);
	            } else if (KEY_REJECTED.equals(key)) {
	            	addMessage(messages, keyPrefix, MessageKeys.REFILL_REJECTED, 
	            		facilities);
	            } else if (KEY_FAILED.equals(key)) {
	            	addMessage(messages, keyPrefix, MessageKeys.SYSTEM_FAILURE, 
	            		facilities);
	            } else  if (KEY_MAX_EXCEEDED.equals(key)) {
	            	addMessage(messages, keyPrefix, MessageKeys.
	            		MAX_FAILURES_EXCEEDED, facilities);
	            }
            } // else: No facility with this 'key' status
        }
        return messages;
    }
    
    private static void addMessage(Messages messages, String keyPrefix, 
    	String key, String facilities) 
   {
    	messages.addMessage(MessageUtils.createInformationalMessage(
    		keyPrefix + key, null, new String[] { facilities }));
    }
	
    private static String toString(Set<String> facilities) {
        StringBuffer stringBuffer = new StringBuffer();
        if (facilities != null) {
            for (String facility: facilities) { 
                if (!StringUtils.isBlank(facility)) {
                    if (stringBuffer.length() > 0) {
                        stringBuffer.append(", ");
                    }
                    stringBuffer.append(facility);
                }
            }
        }
        return stringBuffer.toString();
    }
    
    @SuppressWarnings("unchecked")
	public static List<Request> removeRequestStatusMessages(
        ServiceResponse response) 
    {
        List<Request> requests = new ArrayList<Request>();
        List messagesToRemove = new ArrayList();
        Collection messages = response.getMessages().getInformationalMessages();
        for (Iterator i = messages.iterator(); i.hasNext();) {
            Message message = (Message) i.next();
            if (MESSAGE_KEY_REQUEST_STATUS.equals(message.getKey())) {
            	Iterator inserts = message.getInserts().iterator();
            	if (inserts.hasNext()) {
            		Object object = inserts.next();
            		if (object instanceof Request) {
            			requests.add((Request) object);
            			messagesToRemove.add(message); // unchecked
            		}
            	}
            }
        }
        messages.removeAll(messagesToRemove);  // unchecked
        return requests; 
    }


}
