package gov.va.med.mhv.usermgmt.persist.ldap;

import gov.va.med.mhv.usermgmt.persist.DaoFactory;
import gov.va.med.mhv.usermgmt.persist.DuplicateEntryException;
import gov.va.med.mhv.usermgmt.persist.NoSuchEntryException;
import gov.va.med.mhv.usermgmt.persist.UserEntryDao;

import java.util.Collections;

import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPException;

/**
 * Test relevant update operations performed against the configured directory server.
 *
 * @author Jon Crater Apr 5, 2006 5:22:00 PM
 */
public class LdapUpdateTest extends AbstractLdapTests {
	private UserEntryDao dao;

	public void testCreateUser() throws Exception {
		LDAPEntry entry = readEntry(testUserDn);
		assertNotNull(entry);
		String uid = uidForEntry(entry);
		assertEquals(testUserName, uid);
	}

	public void testRemoveUser() throws Exception {
		dao.removeUser(testUserName);

		try {
			readEntry(testUserDn);
			fail("Expected LDAPException");
		}
		catch(LDAPException e) {
			assertEquals("Expected LDAPException result code to be " + NO_SUCH_OBJECT, NO_SUCH_OBJECT, e.getLDAPResultCode());
		}
	}

	public void testUpdatePassword() throws Exception {
		// userPassword
		LDAPAttribute oldPassAtt = testUserEntry.getAttribute(env.getUserPasswordAttribute());
		String[] oldPassAttValues = oldPassAtt.getStringValueArray();
		assertTrue("Expected only one attribute returned from entry with dn '" + testUserDn + "'", oldPassAttValues.length == 1);
		String newPassword = String.valueOf(System.currentTimeMillis());

		LdapUpdate update = new LdapUpdate(testUserDn);
		update.addCommand(LdapAttributeModification.wipeAndReplace(env.getUserPasswordAttribute(), newPassword));
		update.executeUpdate();

		LDAPEntry newEntry = readEntry(testUserDn);
		LDAPAttribute att = newEntry.getAttribute(env.getUserPasswordAttribute());
		String[] attValues = att.getStringValueArray();

		assertTrue("Expected only one attribute returned from entry with dn '" + testUserDn + "'", attValues.length == 1);

		// would be nice if we could compare actual passwords here, but since they're encrypted,
		// we have to just assert that they're not equal--in essence that a change was made
		assertFalse("Expected passwords not to equal", newPassword.equals(attValues[0]));
	}

	public void testUpdateUserName() throws Exception {
		// this is the new dn that will be created from the dn created in setUp()
		String targetUserName = "newjoeuser";
		String newUserDn = LdapUtil.buildUserDn(targetUserName);

		// make sure this entry doesn't already exist
		try {
			deleteEntry(newUserDn);
		}
		catch(LDAPException e) {
			// don't fail if it doesn't exist
			if(e.getLDAPResultCode() != NO_SUCH_OBJECT)
				throw e;
		}

		// delete and re-create
		LdapUpdate update = new LdapUpdate(newUserDn);
		update.addCommand(LdapEntryModification.wipeAndReplace(testUserDn, Collections.EMPTY_MAP));
		update.executeUpdate();

		// set the testUserDn instance variable equal to the new uid of the entry just created
		// this allows it to be cleaned up in tearDown()
		this.testUserDn = newUserDn;

		// read back the new entry and assert on its attribute values
		LDAPEntry newEntry = readEntry(newUserDn);
		LDAPAttribute newAtt = newEntry.getAttribute(env.getUserNameAttribute());
		String[] newAttValues = newAtt.getStringValueArray();

		// make sure there is only one uid attribute
		assertEquals("Expected only one user id attribute for dn '" + newUserDn + "'", 1, newAttValues.length);
		// make sure the value of the uid attribute on the new entry matches the new uid
		assertEquals(targetUserName, newAttValues[0]);
	}

	public void testNoSuchEntryException() throws Exception {
		// make sure the entry we expect not to exist doesn't exist
		try {
			deleteEntry(testUserDn);
		}
		catch(LDAPException e) {
			if(e.getLDAPResultCode() != LDAPException.NO_SUCH_OBJECT)
				throw e;
		}

		try {
			dao.resetPassword(testUserName, testUserDn);
			fail("Expected an instance of " + NoSuchEntryException.class.getName() + " to be thrown.");
		}
		catch(NoSuchEntryException expected) {
		}
	}

	public void testDuplicateEntryException() throws Exception {
		// make sure the entry we expect to exist does in fact exist
		LDAPEntry entry = readEntry(testUserDn);
		assertNotNull(entry);
		String uid = uidForEntry(entry);
		assertEquals(uid, testUserName);

		try {
			dao.createUser(testUserName, "irrelevant");
			fail("Expected an instance of " + DuplicateEntryException.class.getName() + " to be thrown.");
		}
		catch(DuplicateEntryException expected) {
		}
	}

	public void onSetup() throws Exception {
		dao = (UserEntryDao) DaoFactory.createDao(UserEntryDao.BEAN_NAME);
	}
}